//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  ++39(49) 827-7216     FAX: ++39(49) 827-7102
//     EMail:  loreti@padova.infn.it
//     WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------
//
//     File: main.cxx; main procedure of the program "texref".
//           "texref" reads one or more ".tex" files (gives as
//           arguments), and builds a summary of the used
//           labels.
//     
//     The original program was in C; this C++ version is
//     mainly intended as an exercise.
//     
//     $Id: main.cxx,v 1.9 2006/09/11 12:29:12 loreti Exp $

#include <cstring>
#include "texref.hh"            // Includes: iostream, list, set, string
#include "file.hh"              // Includes: fstream, string

using std::cout;
using std::endl;
using std::set;
using std::string;

// Global variables (definitions)

string        progName;
set<Keyword> *pKeys = new set<Keyword>;
set<Label>   *pLabs = new set<Label>;

// Local procedures

static char  *baseName(char *);
static void   init();
static void   summary();

int main(
  int   argc,
  char *argv[]
) {

// The main procedure.
// - strips in "basename" the path from the program name;
// - checks if file names has been given as parameters; process each
//   one in turn;
// - at the end, prints a list of all what has been found.

  progName = baseName(argv[0]);
  init();

  if (argc < 2) {
    cout << "Usage: " << progName << " file name(s)\n";
  } else {
    while (--argc) {
      string        fileName(*++argv);
      std::ifstream inFile(*argv);

      if (inFile == 0) {
        cout << progName << ": couldn't open file \"" << fileName << "\"\n";
      } else {
        processFile(&inFile, fileName);
      }
    }
    summary();
  }

  delete pKeys;
  delete pLabs;
  return 0;
}

static char *baseName(
  char *pc
) {

// Strips the path part (if any) from the file name

  char *p = std::strrchr(pc, '/');

  if (p == 0) return pc;
  return ++p;
}

static void init()
{

// Keyword initialisation


  static const Keyword kk[] = {
    Keyword("\\bibitem{",   0,   DEFINITION, CITE),
    Keyword("\\cite{",      ",", REFERENCE,  CITE),
    Keyword("\\label{",     0,   DEFINITION, XREF),
    Keyword("\\vref{",      0,   REFERENCE,  XREF),
    Keyword("\\pageref{",   0,   REFERENCE,  XREF),
    Keyword("\\eqref{",     0,   REFERENCE,  XREF),
    Keyword("\\ref{",       0,   REFERENCE,  XREF)
  };

  static const int KEY_NUM = sizeof(kk) / sizeof(kk[0]);
  pKeys->insert(kk, kk+KEY_NUM);
}

static void summary()
{

// Printout of the final summary

  set<Label>::const_iterator pl, plEnd = pLabs->end();

  for (pl = pLabs->begin();   pl != plEnd;   ++pl) {
    std::list<Position>::const_iterator pb, pe;

    cout << "* label " << *(pl->getName()) << endl;

    if (pl->getFam()->size() > 1) {
      cout << "- BELONGS TO MORE THAN ONE LABEL FAMILY\n";
    }

    pb = pl->defBegin();
    pe = pl->defEnd();
    
    if (pb == pe) {
      cout << "- UNDEFINED\n";
    } else {
      string how("defined");
      
      do {
        cout << "- " << how << " in " << *pb << endl;
        how = string("REDEFINED");
      } while (++pb != pe);
    }

    pb = pl->refBegin();
    pe = pl->refEnd();

    if (pb == pe) {
      cout << "- NEVER REFERENCED\n";
    } else {
      do {
        cout << "  referenced in " << *pb << endl;
      } while (++pb != pe);
    }
  }
}

