//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  ++39(49) 827-7216     FAX: ++39(49) 827-7102
//     EMail:  loreti@padova.infn.it
//     WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------
//
//     File: texref.h - Class definitions for "texref".
//   
//     $Id: texref.hh,v 1.2 2001/01/29 07:41:47 loreti Exp $

#ifndef TEXREF_HH_
#define TEXREF_HH_

#include <iostream>
#include <list>
#include <set>
#include <string>

// Constants to identify keyword category and keyword type

typedef enum eKeyFamily { CITE,       XREF }      KeyFamily;
typedef enum eKeyType   { DEFINITION, REFERENCE } KeyType;

// A class for a position inside a file

class Position {

public:
  Position(const std::string & ps, const unsigned long u)
    : _file(ps), _line(u) { }
  Position(const char *pc, const unsigned long u)
    : _file(pc), _line(u) { }
  ~Position() { }
  friend std::ostream & operator << (std::ostream &, const Position &);

private:
  std::string   _file;
  unsigned long _line;
};

// A "keyword", used to define/reference a "label"; if more than
// one "label" may be defined/referenced in a single occurrence,
// the "separator" character are in *_psep; otherwise _psep is NULL.

class Keyword {

public:
  Keyword(const std::string &, const std::string *, const KeyType,
          const KeyFamily);
  Keyword(const char *, const char *, const KeyType, const KeyFamily);
  Keyword(const Keyword &);
  const Keyword & operator = (const Keyword &);
  ~Keyword() { delete _psep; };

  bool operator <  (const Keyword &rhs) const { return (_name <  rhs._name); };
  bool operator == (const Keyword &rhs) const { return (_name == rhs._name); };

  const std::string *getName()   const { return &_name;       };
        size_t       getLen()    const { return _name.size(); };
  const std::string *getSep()    const { return _psep;        };
        KeyType      getType()   const { return _type;        };
        KeyFamily    getFamily() const { return _fam;         };

private:
  std::string  _name;
  std::string *_psep;
  KeyType      _type;
  KeyFamily    _fam;
};

// A label: has a name, and has been defined and referenced

class Label {

public:
  Label(const std::string &);
  Label(const char *);
  Label(const Label &);
  const Label & operator = (const Label &);
  ~Label() { delete _pfam; delete _pdef; delete _pref; };

  bool operator <  (const Label &rhs) const { return (_name <  rhs._name); };
  bool operator == (const Label &rhs) const { return (_name == rhs._name); };

  void addData(const Position pos, const KeyType k, const KeyFamily f) const;

  const std::string         *getName() const { return &_name; };
  const std::set<KeyFamily> *getFam()  const { return _pfam;  };

  std::list<Position>::const_iterator defBegin() const
    { return _pdef->begin(); };
  std::list<Position>::const_iterator defEnd() const
    { return _pdef->end(); };
  std::list<Position>::const_iterator refBegin() const
    { return _pref->begin(); };
  std::list<Position>::const_iterator refEnd() const
    { return _pref->end();   };

private:
  std::string          _name;
  std::set<KeyFamily> *_pfam;
  std::list<Position> *_pdef;
  std::list<Position> *_pref;
};

// Global variables (declarations)

extern std::string        progName;
extern std::set<Keyword> *pKeys;
extern std::set<Label>   *pLabs;

#endif // TEXREF_HH_
