/*------------------------------------------------------*
 | File: texref.c                                       |
 +------------------------------------------------------+
 | 1997-05-31: first release. Scans .tex files building |
 |   cross references between \label and \ref commands. |
 | 1997-06-02: multiple definitions emphasized.         |
 | 1997-06-14: added \eqref (ams) references            |
 | 1997-07-30: added \vref (varioref) references        |
 | 1998-09-03: \cite and \bibitem; the program has been |
 |             rewritten, and mantained under cvs.      |
 +------------------------------------------------------+
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  ++39(49) 827-7216     FAX: ++39(49) 827-7102 |
 | EMail:  loreti@padova.infn.it                        |
 | WWW:    http://wwwcdf.pd.infn.it/~loreti/mlo.html    |
 *------------------------------------------------------*

 $Id: texref.c,v 1.2 2006/09/11 12:51:31 loreti Exp $

 *------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>

#include "texref.h"
#include "file.h"
#include "tree.h"

/**
 | Local global variables: the keyword families.
 | 1) Bibliography
 |    - definition: \bibitem{label} ;
 |    - reference: \cite{label} , or \cite{lab1,lab2,...,labN} .
 | 2) Cross references:
 |    - definition: \label{L} ;
 |    - reference: \ref{L} or \eqref{L} or \pageref{L} or \vref{L} .
**/

static const Keyword kBibitem  = { 9, 0,         "\\bibitem{", 0   };
static const Keyword kCite     = { 6, 0,         "\\cite{",    "," };

static const Keyword kLabel    = { 7, 0,         "\\label{",   0   };
static const Keyword kVref     = { 6, 0,         "\\vref{",    0   };
static const Keyword kPageref  = { 9, &kVref,    "\\pageref{", 0   };
static const Keyword kEqref    = { 7, &kPageref, "\\eqref{",   0   };
static const Keyword kRef      = { 5, &kEqref,   "\\ref{",     0   };

static const LabelClass l2 = {&kBibitem, &kCite, 0};
static const LabelClass l1 = {&kLabel,   &kRef,  &l2};

/**
 | Global variables definition
**/

const LabelClass    *ClassTree = &l1;
      Label         *LabelTree = 0;
      FILE          *pIn       = 0;
const char          *progName;
      char          *fName;
      unsigned long  lineNum;

/**
 | Local (static) procedure prototypes
**/

static const char *basename(const char *);

int main(
  int argc,
  char **argv
) {

/**
 | The main procedure.
 | - strips in "basename" the path from the program name;
 | - checks if file names has been given as parameters; process each
 |   one in turn;
 | - at the end, prints a list of all what has been found.
**/

  progName = basename(argv[0]);

  if (argc < 2) {
    fprintf(stderr, "Usage : %s <file name(s)>\n", progName);
    fputs("\tBuilds a cross-reference table of labels and their\n", stderr);
    fputs("\treferences between the given .tex files.\n", stderr);
    return EXIT_FAILURE;
  }

  while (--argc) {
    if ((pIn = fopen(*++argv, "r")) == 0) {
      fprintf(stderr, "%s: couldn't open file \"%s\"\n", progName, *argv);
      cleanup();
      return EXIT_FAILURE;
    }

    fName   = *argv;
    lineNum = 0;
    dealWithFile();
  }

  listTree();
  cleanup();
  return EXIT_SUCCESS;
}

static const char *basename(
  const char *pc
) {
  char *p;

  p = strrchr(pc, '/');
  if (p == 0) return pc;
  return ++p;
}

void cleanup(void)
{

/**
 | Global cleanup procedure:
 | - closes the input file (if opened);
 | - returns to the operating system all the malloc'd memory (if any).
**/

  if (pIn) {
    fclose(pIn);
  }

  while (LabelTree) {
    Label     *nextLabel = LabelTree->next;
    Reference *pR;
    Reference *nextReference;

    pR = LabelTree->defined;
    while (pR) {
      nextReference = pR->next;
      free(pR);
      pR = nextReference;
    }

    pR = LabelTree->referenced;
    while (pR) {
      nextReference = pR->next;
      free(pR);
      pR = nextReference;
    }

    free(LabelTree);
    LabelTree = nextLabel;
  }
}
