#include <string.h>
#include <stdlib.h>
#include "texref.h"
#include "tree.h"

/**
 | Local (static) procedures
**/

static Label *insertLabel(const char *, const unsigned long, Label *, Label *);
static void   insertNode(const int, Label *, Reference *);

int doTheStuff(
  const int            isDef,   /* Statement type */
  const char          *name,    /* Label name */
  const unsigned long  length   /* Label length */
) {

/**
 | The heart of the program: inserts in the memory tree the information
 | about the occurrence of a given label in the current statement.  In
 | input, "isDef" is TRUE if the label is being defined (and FALSE if it
 | is being referenced; "name" and "length" are the label name and its
 | length.
**/

  Label     *current  = LabelTree;
  Label     *previous = 0;
  Label     *pL;
  Reference *pRef;

/**
 | As the first thing, we need a Reference structure for this occurrence
 | of our label in the input file(s).
**/

  if ((pRef = malloc(sizeof(Reference) + strlen(fName))) == 0) {
    goto merr;
  }

  pRef->lineNo = lineNum;
  pRef->next   = 0;
  strcpy(pRef->fileName, fName);

/**
 | We start looking for the label name in our tree; "current" is the
 | node currently checked, and "previous" its predecessor (we need it,
 | because the linked list will be built alphabetically sorted by label
 | name, in ascending order).  If no previous record is found for the
 | label, a new one is created and inserted in the right place of the
 | linked list; otherwise, the current reference is appended to the
 | existing structure.
**/

  while (current) {
    int cmp;

    if ((cmp = strcmp(name, current->name)) == 0) {
      insertNode(isDef, current, pRef);
      return SUCCESS;
    } else if (cmp < 0) {
      if ((pL = insertLabel(name, length, previous, current)) == 0) {
        goto ferr;
      }

      insertNode(isDef, pL, pRef);
      return SUCCESS;
    }

    previous = current;
    current  = current->next;
  }

/**
 | If we are here, we need to insert a new node to the end of the
 | linked list.
**/

  if ((pL = insertLabel(name, length, previous, current)) == 0) {
    goto ferr;
  }

  insertNode(isDef, pL, pRef);
  return SUCCESS;

/**
 | Error: not enough memory
**/

 ferr:      
  free(pRef);

 merr:
  fprintf(stderr, "%s: not enough memory\n", progName);
  return FAILURE;
}

static Label *insertLabel(
  const char          *name,    /* Label name */
  const unsigned long  length,  /* Label length */
        Label         *prev,    /* Previous node */
        Label         *next     /* Next node */
) {
  Label *pL;

/**
 | Builds a new structure "Label", filled with the label name;
 | then inserts it in the linked list, between "prev" and
 | "next".
**/

  if ((pL = malloc(sizeof(Label) + length)) != 0) {
    pL->defined    = 0;
    pL->referenced = 0;
    strcpy(pL->name, name);

    if (prev) {
      prev->next = pL;
      pL->next   = next;
    } else {
      pL->next  = LabelTree;
      LabelTree = pL;
    }
  }

  return pL;
}

static void insertNode(
  const int        isDef,       /* Statement type */
        Label     *pL,          /* Label we have found */
        Reference *pR           /* Reference to be inserted */
) {
  Reference *pRef;

/**
 | Links to the end of the correct branch (of the label definitions
 | if "isDef" is TRUE, of the label references otherwise) the record
 | of the occurrence of the label in the current line.
**/

  if (isDef) {
    if ((pRef = pL->defined) == 0) {
      pL->defined = pR;
      return;
    }
  } else {
    if ((pRef = pL->referenced) == 0) {
      pL->referenced = pR;
      return;
    }
  }

/**
 | There is at least a leaf in this branch.  Looks forward to find
 | the last leaf, to which appends our reference.
**/

  while (pRef->next) {
    pRef = pRef->next;
  }

  pRef->next = pR;
}

void listTree(void)
{
        Label *pL = LabelTree;
  const char  *pc;

/**
 | Scans the memory tree, printing out the relevant informations.
**/

  while (pL) {
    Reference *pR;

    fprintf(stdout, "* label %s\n", pL->name);

    pR = pL->defined;

    if (pR) {
      pc = "defined";

      while(pR) {
        fprintf(stdout, "- %s in file %s, line %lu\n",
                pc, pR->fileName, pR->lineNo);
        pR = pR->next;
        pc = "REDEFINED";
      }

    } else {
      fprintf(stdout, "- UNDEFINED\n");
    }

    pR = pL->referenced;

    if (pR) {
      while (pR) {
        fprintf(stdout, "  referenced in file %s, line %lu\n",
                pR->fileName, pR->lineNo);
        pR = pR->next;
      }

    } else {
      fprintf(stdout, "  NEVER REFERENCED\n");
    }

    pL = pL->next;
  }
}
